@@ -18,10 +18,12 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #include <sys/ioctl.h>
-#include <linux/if.h> 
+#include <sys/time.h>
+#include <net/if.h> 
 #include <linux/ax25.h>
 #include <stdio.h> 
 #include <stdlib.h>
+#include <string.h>
 #include <syslog.h>
 #include <unistd.h>
 #include <netinet/in.h>
@@ -208,23 +210,29 @@ u_long get_bcast_addr(char *ifname)
  	int skt;
  	struct ifreq ifr;
  	struct sockaddr_in *sin;
- 	
+
+	/* Zero all structures otherwise they will stuff up on you! */
+	bzero((char*)&ifr, sizeof(ifr));
+	 	
  	/* Put the interface name into the ifreq struct */
  	strcpy(ifr.ifr_name, ifname);
  	
 	
  	if ( (skt = socket(AF_INET, SOCK_DGRAM,0)) < 0) {
  		syslog(LOG_DAEMON | LOG_ERR, "get_bcast_addr(): socket failed. (%m)");
- 		(void) close(skt);
+ 		if (close(skt) < 0)
+ 			syslog(LOG_DAEMON | LOG_ERR, "get_bcast_addr(): close failed (%m)");
  		return INADDR_NONE;
  	}
  	if (ioctl(skt, SIOCGIFBRDADDR, &ifr) < 0) {
  		syslog(LOG_DAEMON | LOG_ERR, "get_bcast_addr(): ioctl failed for %s. (%m)", ifname);
- 		(void) close(skt);
+ 		if (close(skt) < 0)
+ 			syslog(LOG_DAEMON | LOG_ERR, "get_bcast_addr(): close failed (%m)");
  		return INADDR_NONE;
  	}
 	
-	(void) close(skt);
+	if (close(skt) < 0)
+ 			syslog(LOG_DAEMON | LOG_ERR, "get_bcast_addr(): close failed (%m)");
 	
 	sin = (struct sockaddr_in*)&ifr.ifr_broadaddr;
 	return sin->sin_addr.s_addr;
@@ -245,25 +253,38 @@ u_long get_iface_addr(char *ifname)
 {
  	int skt;
  	struct ifreq ifr;
+ 	u_long addr;
+ 	
  	struct sockaddr_in *sin;
  	
+ 	bzero((char*)&ifr, sizeof(ifr));
+ 	
  	/* Put the interface name into the ifreq struct */
  	strcpy(ifr.ifr_name, ifname);
  	
  	if ( (skt = socket(AF_INET, SOCK_DGRAM,0)) < 0) {
  		syslog(LOG_DAEMON | LOG_ERR, "get_iface_addr(): socket failed for %s. (%m)", ifname);
- 		(void) close(skt);
+		if (close(skt) < 0)
+ 			syslog(LOG_DAEMON | LOG_ERR, "get_iface_addr(): close failed (%m)");
  		return INADDR_NONE;
  	}
+	(void) ioctl(skt, SIOCGIFFLAGS, &ifr);
+	
  	if (ioctl(skt, SIOCGIFADDR, &ifr) < 0) {
  		syslog(LOG_DAEMON | LOG_ERR, "get_iface_addr(): ioctl failed. (%m)");
- 		(void) close(skt);
+		if (close(skt) < 0)
+ 			syslog(LOG_DAEMON | LOG_ERR, "get_iface_addr(): close failed (%m)");
  		return INADDR_NONE;
  	}
+	if (close(skt) < 0)
+		syslog(LOG_DAEMON | LOG_ERR, "get_iface_addr(): close failed (%m)");	
 	
-	(void) close(skt);
-	sin = (struct sockaddr_in*)&ifr.ifr_addr;
-	return sin->sin_addr.s_addr;
+	sin = (struct sockaddr_in*)&(ifr.ifr_addr);
+/*	printf("get_iface_addr(): iface %s address is %s.\n", ifname, inet_ntoa(sin->sin_addr) );*/
+	
+	bcopy((char*)&(sin->sin_addr.s_addr), (char*)&addr, 4);
+
+	return addr;
 	
 }	
 
@@ -277,4 +298,4 @@ void clear_all_ifaces()
 	
 	rspf_stats.rspfCurrIfaces = 0;
 		
-}
\ No newline at end of file
+}
