@@ -19,19 +19,22 @@
 #include <arpa/inet.h>
 #include <linux/ax25.h>
 #include <netinet/in.h>
-#include <net/if_route.h>
+#include <net/route.h>
 #include <net/if.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <sys/time.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "rspfd.h"
 #include "rspfroute.h"
 #include "queue.h"
+#include "rspfif.h"
 
 extern struct queue *adj_queue;
 extern struct queue *rt_queue;
@@ -63,18 +66,32 @@ void del_route(u_long daddr, u_char sigbits)
  	
  	if ( (skt = socket(AF_INET, SOCK_DGRAM,0)) < 0) 
 	{
- 		syslog(LOG_DAEMON | LOG_ERR, "add_route(): socket failed. (%m)");
- 		(void) close(skt);
+ 		syslog(LOG_DAEMON | LOG_ERR, "del_route(): socket failed. (%m)");
+ 		if (close(skt) < 0)
+ 			syslog(LOG_DAEMON | LOG_ERR, "del_route(): close failed. (%m)");
  		return;
  	}
  	
  	if (ioctl(skt, SIOCDELRT, &rte) < 0)
  	{
- 		syslog(LOG_DAEMON | LOG_ERR, "del_route(): ioctl failed with address %s. (%m)", in_ntoa(daddr));
- 		close(skt);
- 		return;
+ 		/* Check for 'no such process'. This means that the route is not in the
+ 		 * table, talk about undocumented features!! (At least in *my* man 
+ 		 * pages)
+ 		 */
+		if (errno == ESRCH)
+		{
+			syslog(LOG_DAEMON | LOG_WARNING, "del_route(): ioctl failed because %s is not in kernel's table.", in_ntoa(daddr));
+		}			
+		else
+		{						 		
+	 		syslog(LOG_DAEMON | LOG_ERR, "del_route(): ioctl failed with address %s. (%m)", in_ntoa(daddr));
+ 			if (close(skt) < 0)
+ 				syslog(LOG_DAEMON | LOG_ERR, "del_route(): close failed. (%m)");
+	 		return;
+	 	}
  	}
- 	close(skt);
+	if (close(skt) < 0)
+		syslog(LOG_DAEMON | LOG_ERR, "del_route(): close failed. (%m)");
  	rt = (struct rspf_route*)qmove_first(rt_queue, &rtqm);
  	while(rt != NULL)
  	{
@@ -97,8 +114,9 @@ void nuke_routes()
 	
 	if ( (skt = socket(AF_INET, SOCK_DGRAM,0)) < 0) 
 	{
- 		syslog(LOG_DAEMON | LOG_ERR, "del_route(): socket failed. (%m)");
- 		(void) close(skt);
+ 		syslog(LOG_DAEMON | LOG_ERR, "nuke_routes(): socket failed. (%m)");
+		if (close(skt) < 0)
+			syslog(LOG_DAEMON | LOG_ERR, "nuke_routes(): close failed. (%m)");
  		return;
  	}
  	
@@ -109,15 +127,24 @@ void nuke_routes()
 		bcopy((char*)&rt->addr, &rte.rt_dst, 4);
 	 	if (ioctl(skt, SIOCDELRT, &rt) < 0)
  		{
- 			syslog(LOG_DAEMON | LOG_ERR, "del_route(): ioctl failed. (%m)");
-	 		close(skt);
- 			return;
+ 			if (errno == ESRCH)
+			{
+				syslog(LOG_DAEMON | LOG_WARNING, "nuke_routes(): ioctl failed because %s is not in kernel's table.", in_ntoa(rt->addr));
+			}			
+			else
+			{						 		
+	 			syslog(LOG_DAEMON | LOG_ERR, "nuke_routes(): ioctl failed, cannot delete %s. (%m)", in_ntoa(rt->addr));
+				if (close(skt) < 0)
+					syslog(LOG_DAEMON | LOG_ERR, "nuke_routes(): close failed. (%m)");
+ 				return;
+ 			}
 	 	}	
 		del_qnode(rt_queue, rtqm, 1);
 		
 		rt = (struct rspf_route*)qmove_first(rt_queue, &rtqm);
 	}
-	close(skt);
+	if (close(skt) < 0)
+		syslog(LOG_DAEMON | LOG_ERR, "nuke_routes(): close failed. (%m)");
 } /* nuke_routes() */	
 
 struct rtentry *get_route(u_long addr, u_char sigbits)
@@ -132,7 +159,7 @@ struct rtentry *get_route(u_long addr, u_char sigbits)
 	
 	if ( (fp = fopen("/proc/net/route", "r")) == NULL)
 	{
-		syslog(LOG_ERR | LOG_DAEMON, "get_route_cost(): Cannot open proc route file (%m)");
+		syslog(LOG_ERR | LOG_DAEMON, "get_route(): Cannot open proc route file (%m)");
 		return NULL;
 	}
 	bzero ((char*)&rt, sizeof(rt));
@@ -149,9 +176,9 @@ struct rtentry *get_route(u_long addr, u_char sigbits)
 	
 	while(fgets(buf, 256, fp) != NULL)
 	{
-		if (sscanf(buf, "%20s %lX %lX %hX %hd %lu %hd %lX %lu %lu", 
-			rt.rt_dev,&daddr, &gaddr, &rt.rt_flags, &rt.rt_refcnt,
-			&rt.rt_use, &rt.rt_metric, &maddr, &rt.rt_mss, &rt.rt_window) != 10)
+		if (sscanf(buf, "%20s %lX %lX %hX %*d %*u %hd %lX %lu %lu %hu", 
+			rt.rt_dev,&daddr, &gaddr, &rt.rt_flags, 
+			&rt.rt_metric, &maddr, &rt.rt_mss, &rt.rt_window, &rt.rt_irtt) != 9)
 			
 			continue;
 
@@ -207,11 +234,16 @@ void add_route(u_long daddr, u_char sigbits, u_long gateway, u_char cost)
 	struct rtentry *oldrt;
 	struct rspf_route *rt;
 	struct nodegroup *ng;
-	qmark rtqm, adj_qm, ng_qm;
+	qmark rtqm, ng_qm;
 	struct sockaddr_in *sin;
 	int skt;
 	char key[KEY_SIZE];
+	u_long next_addr;
 	char iface[IFNAMSIZ];
+	u_short irtt = 0;
+	char ifbuf[128];
+	char *ifname;
+	int ifcount;
 	struct rspf_adj *aptr;
 	
 	if (debug_mode)
@@ -219,6 +251,24 @@ void add_route(u_long daddr, u_char sigbits, u_long gateway, u_char cost)
 		printf("add_route(): addr = %s/%u  ", in_ntoa(daddr), sigbits);
 		printf(" gate = %s  cost %u.\n", (gateway? in_ntoa(gateway): "NONE"),  cost);	
 	}
+
+	/*
+	 * See if this is a route to us.  This can happen if another router
+	 * has a links to us.  do_spf() should catch such things, but it
+	 * doesn't yet.
+	 */
+	ifcount = rspf_ifaces(ifbuf, 128);
+	ifname = ifbuf;
+	while(ifcount-- > 0)
+	{
+		if (get_iface_addr(ifname) == daddr) {
+			if (debug_mode)
+				printf("Not adding route to myself!\n");
+			return;
+		}
+		ifname += strlen(ifname) + 1;
+	}
+
 	/*
 	 * See if we already have better route
 	 * Currently, the definition of this is a route without a dynamic
@@ -227,8 +277,16 @@ void add_route(u_long daddr, u_char sigbits, u_long gateway, u_char cost)
 	 */
 	if ( (oldrt = get_route(daddr, sigbits)) != NULL )
 	{
-/*		if (oldrt->rt_metric < cost)
-			return;*/
+/* The kernel supports rt_metric, but this breaks */
+/* the ability to change cost of an adjacency!    */
+#ifdef notdef
+		if (oldrt->rt_metric < cost)
+		{
+			if (debug_mode)
+				printf("Not adding route, already have better metric (%u).\n",oldrt->rt_metric);
+			return;
+		}
+#endif
 		if (!(oldrt->rt_flags & RTF_DYNAMIC))
 		{
 			if (debug_mode)
@@ -241,10 +299,10 @@ void add_route(u_long daddr, u_char sigbits, u_long gateway, u_char cost)
 	 * Now try to find what interface to use
 	 */
 	if (gateway)
-		strcpy(key, in_ntoa(gateway));
+		next_addr = gateway;
 	else
-		strcpy(key, in_ntoa(daddr));
-		
+		next_addr = daddr;
+				
 	ng = (struct nodegroup*)qmove_first(nodgrpq, &ng_qm);
 	while(ng != NULL)
 	{
@@ -253,9 +311,10 @@ void add_route(u_long daddr, u_char sigbits, u_long gateway, u_char cost)
 			
 		ng = (struct nodegroup*)qmove_next(nodgrpq, &ng_qm);
 	}
-	if ( (aptr = (struct rspf_adj*)qfind_first(adj_queue, key, &adj_qm)) != NULL)
+	if ( (aptr = getadjbyip(next_addr)) != NULL)
 	{
 		strncpy(iface, aptr->port, IFNAMSIZ);
+		irtt = aptr->irtt;
 	}
 	else 
 	{
@@ -309,6 +368,11 @@ void add_route(u_long daddr, u_char sigbits, u_long gateway, u_char cost)
 		rte.rt_flags |= RTF_HOST;
 	if (gateway)
 		rte.rt_flags |= RTF_GATEWAY;
+	if (irtt > 0) 
+	{
+		rte.rt_flags |= RTF_IRTT;
+		rte.rt_irtt = irtt;
+	}
 	
 	rte.rt_metric =	cost;
 	
@@ -318,16 +382,19 @@ void add_route(u_long daddr, u_char sigbits, u_long gateway, u_char cost)
  	if ( (skt = socket(AF_INET, SOCK_DGRAM,0)) < 0) 
 	{
  		syslog(LOG_DAEMON | LOG_ERR, "add_route(): socket failed. (%m)");
- 		(void) close(skt);
+		if (close(skt) < 0)
+			syslog(LOG_DAEMON | LOG_ERR, "add_route(): close failed. (%m)");
  		return;
  	} 	
  	if (ioctl(skt, SIOCADDRT, &rte) < 0)
  	{
  		syslog(LOG_DAEMON | LOG_ERR, "add_route(): ioctl failed. (%m)");
- 		close(skt);
+		if (close(skt) < 0)
+			syslog(LOG_DAEMON | LOG_ERR, "add_route(): close failed. (%m)");
  		return;
  	}
- 	close(skt);
+	if (close(skt) < 0)
+		syslog(LOG_DAEMON | LOG_ERR, "add_route(): close failed. (%m)");
  	/*
  	 * Add route to internal table
  	 */
@@ -358,4 +425,4 @@ void add_route(u_long daddr, u_char sigbits, u_long gateway, u_char cost)
  	return;
 } /* add_route() */
 
-	
\ No newline at end of file
+
